﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Transactions;
using com.cruxomatic.DTOs;
using com.cruxomatic.BusinessObjects;
using com.cruxomatic.configuration;
using com.cruxomatic.security.EncryptionAndHashing;
using com.cruxomatic.security.EncryptionAndHashing.Enums;
using com.cruxomatic.BusinessObjects.DatabaseQueries;
using com.cruxomatic.notifications;
using com.cruxomatic.logging;
using com.cruxomatic.logging.Enums;
using com.cruxomatic.exceptions;

namespace com.cruxomatic.BusinessFacades
{
    public class UserFacade
    {
        public List<UserInformation> GetUsers(int CompanyID)
        {
            Queries usersQuery = new Queries();
            IEnumerable<UserInformation> users =
                from usersList in usersQuery.GetUsers(CompanyID)
                select ClassConverter.CopyUserToUserInformation(usersList);
            return users.ToList();
        }

        public UserInformation GetUser(int UserID)
        {
            if (UserID == 0) return new UserInformation();
            Queries userQuery = new Queries();
            User user = userQuery.GetUser(UserID);
            return ClassConverter.CopyUserToUserInformation(user);
        }

        public UserInformation GetUser(string UserName)
        {
            Queries userQuery = new Queries();
            User user = userQuery.GetUser(UserName);
            return ClassConverter.CopyUserToUserInformation(user);
        }

        public void SaveUser(UserInformation user, PasswordStorageType passwordstoragetype, bool isFeatureSupported)
        {
            CheckValidity(user);
            Queries userQuery = new Queries();
            User dbUser = new User();
            if (user.UserID != 0)
                dbUser = userQuery.GetUser(user.UserID);

            dbUser = ClassConverter.CopyUserInformationToUser(user, dbUser);
            if (user.UserID != 0)
                userQuery.SubmitChanges();
            else
            {
                CreateUser(user, dbUser, userQuery, passwordstoragetype, isFeatureSupported);
                
            }
        }

        private void CheckValidity(UserInformation user)
        {
            if (UserExistInCompany(user))
                throw CruxExceptionManager.CreateNewDatabaseException("905");
        }

        public bool UserExistInCompany(UserInformation user)
        {
            Queries query = new Queries();
            User dbUser = query.GetUserFromCompany(user.UserName);
            if (dbUser != null && dbUser.UserID != user.UserID)
                return true;
            return false;
        }

        private void CreateUser(UserInformation user, User dbUser, Queries userQuery, PasswordStorageType passwordStorageType, bool isFeatureSupported)
        {
            KeyValuePair<string, string> RandomAndHashedPassword = GenerateRandomPassword(passwordStorageType);
            string RandomPassword = RandomAndHashedPassword.Key;
            using (TransactionScope scope = new TransactionScope())
            {
                dbUser.Password = RandomAndHashedPassword.Value;
                dbUser.IsResetPasswordNeeded = true;
                dbUser.PasswordChangedOn = DateTime.Now;
                UserCompany usercompany = new UserCompany();
                usercompany.User = dbUser;
                usercompany.CompanyID = dbUser.CompanyID;
                dbUser.UserCompanies.Add(usercompany);
                try
                {
                    userQuery.InsertUser(dbUser);
                    if (isFeatureSupported)
                        SaveNewPassword(dbUser, userQuery);
                    userQuery.SubmitChanges();
                    scope.Complete();
                }
                catch (Exception ex)
                {
                    scope.Dispose();
                    throw ex;
                }
            }
            try
            {
                SendCreateUserNotificationToUser(user, RandomPassword);
            }
            catch (Exception ex)
            {
                ApplicationLogger.LogApplicationMessage(ex.Message, ApplicationLogType.Notification, ex.StackTrace);
                throw CruxExceptionManager.CreateNewNotificationException("701");
            }

        }

        public void SendCreateUserNotificationToUser(UserInformation user, string RandomPassword)
        {
            List<KeyValuePair<string, string>> TagsAndValues = new List<KeyValuePair<string, string>>();
            TagsAndValues.Add(new KeyValuePair<string, string>("FullName", user.FullName));
            TagsAndValues.Add(new KeyValuePair<string, string>("UserName", user.UserName));
            TagsAndValues.Add(new KeyValuePair<string, string>("Password", RandomPassword));

            List<string> recepients = new List<string>();
            recepients.Add(user.UserEmailAddress);

            MailMan mailmail = new MailMan(CruxConfigurationManager.GetFullConifgurationSettings());
            mailmail.SendMail("CreateUser", TagsAndValues, recepients);
        }

        private void SaveNewPassword(User dbUser, Queries query)
        {
            ChangedPassword changedPassword = new ChangedPassword();
            changedPassword.Password = dbUser.Password;
            changedPassword.PasswordChangedOn = dbUser.PasswordChangedOn;
            changedPassword.UserID = dbUser.UserID;
            query.InsertPassword(changedPassword);
        }

        private KeyValuePair<string, string> GenerateRandomPassword(PasswordStorageType passwordStorageType)
        {
            //TODO: Improove Password Generation Mechanism. First Six Characters Of Guid Are A Neat Idea
            //but find out how the rest of the world does it. .NET is supposed to have some API for this
            //becuase the membership providers renegrate forgotten passwords. 

            Guid randomKey = Guid.NewGuid();
            string randomPassword = randomKey.ToString().Substring(0, 6);
            string randomHashedPassword = randomPassword;
            if (passwordStorageType == PasswordStorageType.hashed)
                randomHashedPassword = Hasher.GetHash(randomPassword);
            KeyValuePair<string, string> NormalAndHashedPassword = new KeyValuePair<string, string>(randomPassword, randomHashedPassword);
            return NormalAndHashedPassword;
        }

        public void DeleteUser(int UserID)
        {
            Queries userQuery = new Queries();
            User user = userQuery.GetUser(UserID);
            userQuery.DeleteUser(user);
        }

        public List<UserInformation> GetAllUsers()
        {
            Queries usersQuery = new Queries();
            IEnumerable<UserInformation> users =
                from usersList in usersQuery.GetAllUsers()
                select ClassConverter.CopyUserToUserInformation(usersList);
            return users.ToList();
        }

        public List<CompanyInformation> GetCompaniesForUser(int userid)
        {
            Queries userQuery = new Queries();
            IEnumerable<CompanyInformation> companies = userQuery.GetCompaniesForUser(userid);
            return companies.ToList();
        }


        public void AssignCompaniesToUser(int userid, List<CompanyInformation> selectedCompanies)
        {
            Queries query = new Queries();
            IEnumerable<UserCompany> usercompanies = query.GetUserCompanyForUser(userid);
            foreach (var userCompany in usercompanies)
            {
                query.DeleteCompanyAssociationForUser(userCompany, false);
            }

            foreach (CompanyInformation companyInformation in selectedCompanies)
            {
                UserCompany usercompany = new UserCompany() { UserID = userid, CompanyID = companyInformation.CompanyID };
                query.AssignUserToCompany(usercompany, false);
            }
            query.SubmitChanges();
        }

        public List<CompanyInformation> GetCompaniesForUser(string UserName)
        {
            int userid = GetUser(UserName).UserID;
            return GetCompaniesForUser(userid);
        }

        public void SetLastLoginTime(int userId)
        {
            Queries queries = new Queries();
            User dbUser = queries.GetUser(userId);
            dbUser.LastLoginTime = DateTime.Now;
            queries.SubmitChanges();
        }
    }
}
